@@ -11,21 +11,26 @@ import Control.Monad.State (liftIO)
 import System.Posix.Files
 
 import Command
-import qualified Annex
+import qualified AnnexQueue
 import qualified Backend
 import LocationLog
 import Types
-import Core
+import Content
 import Messages
+import Utility
+import Touch
+
+command :: [Command]
+command = [repoCommand "add" paramPath seek "add files to annex"]
 
 {- Add acts on both files not checked into git yet, and unlocked files. -}
-seek :: [SubCmdSeek]
+seek :: [CommandSeek]
 seek = [withFilesNotInGit start, withFilesUnlocked start]
 
 {- The add subcommand annexes a file, storing it in a backend, and then
  - moving it into the annex directory and setting up the symlink pointing
  - to its content. -}
-start :: SubCmdStartBackendFile
+start :: CommandStartBackendFile
 start pair@(file, _) = notAnnexed file $ do
 	s <- liftIO $ getSymbolicLinkStatus file
 	if (isSymbolicLink s) || (not $ isRegularFile s)
@@ -34,19 +39,26 @@ start pair@(file, _) = notAnnexed file $ do
 			showStart "add" file
 			return $ Just $ perform pair
 
-perform :: BackendFile -> SubCmdPerform
+perform :: BackendFile -> CommandPerform
 perform (file, backend) = do
 	stored <- Backend.storeFileKey file backend
 	case stored of
 		Nothing -> return Nothing
-		Just (key, _) -> return $ Just $ cleanup file key
+		Just (key, _) -> do
+			moveAnnex key file
+			return $ Just $ cleanup file key
 
-cleanup :: FilePath -> Key -> SubCmdCleanup
+cleanup :: FilePath -> Key -> CommandCleanup
 cleanup file key = do
-	moveAnnex key file
 	logStatus key ValuePresent
 
 	link <- calcGitLink file key
 	liftIO $ createSymbolicLink link file
-	Annex.queue "add" ["--"] file
+
+	-- touch the symlink to have the same mtime as the file it points to
+	s <- liftIO $ getFileStatus file
+	let mtime = modificationTime s
+	liftIO $ touch file (TimeSpec mtime) False
+
+	AnnexQueue.add "add" [Param "--"] file
 	return True
