@@ -9,36 +9,36 @@ module Command.DropKey where
 
 import Command
 import qualified Annex
-import qualified Backend
 import LocationLog
 import Types
-import Core
+import Content
 import Messages
 
-seek :: [SubCmdSeek]
+command :: [Command]
+command = [repoCommand "dropkey" (paramRepeating paramKey) seek
+	"drops annexed content for specified keys"] 
+
+seek :: [CommandSeek]
 seek = [withKeys start]
 
-{- Drops cached content for a key. -}
-start :: SubCmdStartString
-start keyname = do
-	backends <- Backend.list
-	let key = genKey (head backends) keyname
+start :: CommandStartKey
+start key = do
 	present <- inAnnex key
-	force <- Annex.flagIsSet "force"
+	force <- Annex.getState Annex.force
 	if not present
 		then return Nothing
 		else if not force
 			then error "dropkey is can cause data loss; use --force if you're sure you want to do this"
 			else do
-				showStart "dropkey" keyname
+				showStart "dropkey" (show key)
 				return $ Just $ perform key
 
-perform :: Key -> SubCmdPerform
+perform :: Key -> CommandPerform
 perform key = do
 	removeAnnex key
 	return $ Just $ cleanup key
 
-cleanup :: Key -> SubCmdCleanup
+cleanup :: Key -> CommandCleanup
 cleanup key = do
 	logStatus key ValueMissing
 	return True
