@@ -11,13 +11,17 @@ import Control.Monad (when)
 import Control.Monad.State (liftIO)
 
 import Command
-import Core
+import Content
 
-seek :: [SubCmdSeek]
-seek = [withDefault "." withFilesInGit start]
+command :: [Command]
+command = [repoCommand "find" (paramOptional $ paramRepeating paramPath) seek
+	"lists available files"]
+
+seek :: [CommandSeek]
+seek = [withFilesInGit start]
 
 {- Output a list of files. -}
-start :: SubCmdStartString
+start :: CommandStartString
 start file = isAnnexed file $ \(key, _) -> do
 	exists <- inAnnex key
 	when exists $ liftIO $ putStrLn file
