@@ -10,39 +10,41 @@ module Command.FromKey where
 import Control.Monad.State (liftIO)
 import System.Posix.Files
 import System.Directory
-import Control.Monad (when, unless)
+import Control.Monad (unless)
 
 import Command
-import qualified Annex
+import qualified AnnexQueue
 import Utility
 import qualified Backend
-import Types
-import Core
+import Content
 import Messages
+import Key
 
-seek :: [SubCmdSeek]
+command :: [Command]
+command = [repoCommand "fromkey" paramPath seek
+	"adds a file using a specific key"]
+
+seek :: [CommandSeek]
 seek = [withFilesMissing start]
 
-{- Adds a file pointing at a manually-specified key -}
-start :: SubCmdStartString
-start file = do
-	keyname <- Annex.flagGet "key"
-	when (null keyname) $ error "please specify the key with --key"
-	backends <- Backend.list
-	let key = genKey (head backends) keyname
-
+start :: CommandStartString
+start file = notBareRepo $ do
+	key <- cmdlineKey
 	inbackend <- Backend.hasKey key
 	unless inbackend $ error $
-		"key ("++keyname++") is not present in backend"
+		"key ("++keyName key++") is not present in backend"
 	showStart "fromkey" file
-	return $ Just $ perform file key
-perform :: FilePath -> Key -> SubCmdPerform
-perform file key = do
+	return $ Just $ perform file
+
+perform :: FilePath -> CommandPerform
+perform file = do
+	key <- cmdlineKey
 	link <- calcGitLink file key
 	liftIO $ createDirectoryIfMissing True (parentDir file)
 	liftIO $ createSymbolicLink link file
 	return $ Just $ cleanup file
-cleanup :: FilePath -> SubCmdCleanup
+
+cleanup :: FilePath -> CommandCleanup
 cleanup file = do
-	Annex.queue "add" ["--"] file
+	AnnexQueue.add "add" [Param "--"] file
 	return True
