@@ -14,22 +14,26 @@ import Command
 import Messages
 import qualified Annex
 import qualified GitRepo as Git
+import Utility
+	
+command :: [Command]
+command = [repoCommand "lock" paramPath seek "undo unlock command"]
 
-seek :: [SubCmdSeek]
+seek :: [CommandSeek]
 seek = [withFilesUnlocked start]
 
 {- Undo unlock -}
-start :: SubCmdStartBackendFile
+start :: CommandStartBackendFile
 start (file, _) = do
 	showStart "lock" file
 	return $ Just $ perform file
 
-perform :: FilePath -> SubCmdPerform
+perform :: FilePath -> CommandPerform
 perform file = do
 	liftIO $ removeFile file
 	g <- Annex.gitRepo
 	-- first reset the file to drop any changes checked into the index
-	liftIO $ Git.run g ["reset", "-q", "--", file]
+	liftIO $ Git.run g "reset" [Params "-q --", File file]
 	-- checkout the symlink
-	liftIO $ Git.run g ["checkout", "--", file]
+	liftIO $ Git.run g "checkout" [Param "--", File file]
 	return $ Just $ return True -- no cleanup needed
