@@ -20,15 +20,19 @@ import qualified Annex
 import qualified Command.Unannex
 import qualified Command.Init
 
-seek :: [SubCmdSeek]
-seek = [withAll withFilesInGit Command.Unannex.start, withNothing start]
+command :: [Command]
+command = [repoCommand "uninit" paramPath seek 
+        "de-initialize git-annex and clean out repository"]
 
-start :: SubCmdStartNothing
+seek :: [CommandSeek]
+seek = [withFilesInGit Command.Unannex.start, withNothing start]
+
+start :: CommandStartNothing
 start = do
 	showStart "uninit" ""
 	return $ Just $ perform
 
-perform :: SubCmdPerform
+perform :: CommandPerform
 perform = do
 	g <- Annex.gitRepo
 
@@ -55,5 +59,5 @@ gitAttributesUnWrite repo = do
 	attrexists <- doesFileExist attributes
 	when attrexists $ do
 		c <- readFileStrict attributes
-		writeFile attributes $ unlines $
-			filter (/= Command.Init.attrLine) $ lines c
+		safeWriteFile attributes $ unlines $
+			filter (\l -> not $ l `elem` Command.Init.attrLines) $ lines c
