@@ -7,18 +7,25 @@
 
 module CopyFile (copyFile) where
 
+import Control.Monad (when)
+import System.Directory (doesFileExist, removeFile)
+
 import Utility
 import qualified SysConfig
 
 {- The cp command is used, because I hate reinventing the wheel,
  - and because this allows easy access to features like cp --reflink. -}
 copyFile :: FilePath -> FilePath -> IO Bool
-copyFile src dest = boolSystem "cp" opts
+copyFile src dest = do
+	e <- doesFileExist dest
+	when e $
+		removeFile dest
+	boolSystem "cp" [params, File src, File dest]
 	where
-		opts = if SysConfig.cp_reflink_auto
-			then ["--reflink=auto", src, dest]
+		params = if SysConfig.cp_reflink_auto
+			then Params "--reflink=auto"
 			else if SysConfig.cp_a
-				then ["-a", src, dest]
+				then Params "-a"
 				else if SysConfig.cp_p
-					then ["-p", src, dest]
-					else [src, dest]
+					then Params "-p"
+					else Params ""
